/*
    This file is part of serialreader, an HTML reader for the iRex Iliad,
    and is copyright (c) 2010 - 2012 Volker Schatz.

    It is licensed under Version 3 or later of the Gnu General Public License
    (see http://www.gnu.org/licenses/gpl.html for details).
*/

#ifndef SERIALREADER_H
#define SERIALREADER_H

#include <stdio.h>

/* General stuff: */

#define CONFIG_NAME     "serialreader.cfg"
#ifndef SR_RENDER
#define GLOBAL_CONFIG   "/etc/serialreader.cfg"
#else
#define GLOBAL_CONFIG   "/data/iliad/homebrew/readerprogs/serialreader/sr_render.cfg"
#endif

#ifndef SCREEN_WIDTH
#define SCREEN_WIDTH 768
#define SCREEN_HEIGHT 1024
#define CLIENT_AREA (SCREEN_HEIGHT - 89)
#endif

#define min2(x,y)   ((x)>(y)?(y):(x))
#define max2(x,y)   ((x)>(y)?(x):(y))

typedef signed char schar;
typedef unsigned char uchar;
typedef unsigned short ushort;

#define CHAR_PRINT_MIN  0x1F
#define CHAR_ERASED     0x18
#define CHAR_STOP       0x1E
#define CHAR_PARAGRAPH  0x1D
#define CHAR_UNKNOWN    CHAR_PRINT_MIN

#define STYLE_REGULAR   0x11
#define STYLE_BOLD      0x12
#define STYLE_ITALIC    0x13
#define STYLE_MONO      0x14

#define isstyle(c)      ((c) >= 0x11 && (c) <= 0x14)

#define NSTYLES     4

typedef struct {
    char *pos;
    int style;
}
text_pos;

/* From basics.c : */

typedef struct {
    void *buf;
    unsigned size;
    unsigned increment;
}
autospace;

typedef struct {
    void *buf;
    unsigned totalsize;
    unsigned reserved0;
    unsigned elemsize, nelem, maxelem;
}
autoarray;

void autoalloc(autospace *sp);
void autoextend(autospace *sp);
void autoappend(autoarray *ary, const void *elem);
FILE *stddiag(void);
char *catpath(const char *path, const char *file);
void *load_file(const char *filename, long *filesize);
int strcase2cmp(const void *s1, const void *s2);
int wordcmp(const char *s1, const char *s2);
char *afterwords(const char *haystack, const char *lcwords);

/* From config.c : */

#define MAXFONTS    (4*NSTYLES)

typedef struct {
    char *fontfiles[MAXFONTS];
    int numfontsizes, fontsize, margin;
    char *defaultencoding, *forceencoding;
    char *start_file, *argv_file, *docdir;
    int start_position;
}
sr_config;

extern sr_config config;

int read_config(void);
int write_config(void);

/* From unicode.c : */

int valid_utf8(const char *text, const char **next);
int utf8_codepoint(const char *text, const char **next);
int utf8_isspace(const char *text, const char **next);
void utf8_write(int codepoint, char *dest, char **end);
void utf8_skip(const char **text);
void utf8_rskip(const char **text);

/* From font.c : */

#pragma pack(1)

typedef struct {
    uchar height;
    uchar width;
    schar BearingX;
    schar BearingY;
    uchar Advance;
}
smallGlyphMetrics;

/* Workaround for arm-linux-gcc, which rounds sizeof() up to multiples of 4 */
#define SIZEOF_smallGlyphMetrics    5

#pragma pack()

typedef struct {
    smallGlyphMetrics *metrics;
    char *bitmap;
    int bitsize, bitpacked, advance;
}
otb_glyph;

int load_all_fonts(void);
const otb_glyph *get_glyph(int codepoint, int style);
int have_glyph(int codepoint);
void line_height(const text_pos *line, int *height, int *depth, text_pos *next);
int parskip(void);

/* From encodings.c : */

typedef struct {
    char *name;
    unsigned *utf8table;
}
encoding;

extern encoding encodings8bit[];

/* From document.c : */

typedef struct {
    char *filename;
    char *text;
    autoarray lines;
    int curr_first_line, curr_last_line;
}
sr_document;

sr_document *load_document(char *filename);

/* From linebreak.c : */

void break_document(char *text, autoarray *linelist);

/* From render.c : */

void typeset_page(sr_document *doc, int pagestep);
int read_posbar(sr_document *doc, int x);
int near_posbar(int x, int y);

/* From navigation.c: */

#define NAV_INIT   0
#define NAV_FIRST  1
#define NAV_PREV   2
#define NAV_NEXT   3
#define NAV_LAST   4
#define NAV_SET    5

extern sr_document *document;

int to_docdir(char *argv1);
int numdocs(void);
int currdoc(void);
int change_file(int mode);
void change_page(int mode);
void posbar_page(int x);
int have_history(int mode);
int nav_history(int mode);
int can_change_fontsize(int mode);
int change_fontsize(int mode);

/* From Iliad-specific sources: */

extern unsigned char *screen;

#endif

