/*
    This file is part of serialreader, an HTML reader for the iRex Iliad,
    and is copyright (c) 2010 - 2012 Volker Schatz.

    It is licensed under Version 3 or later of the Gnu General Public License
    (see http://www.gnu.org/licenses/gpl.html for details).
*/

/*
    Test program which renders a page.
gcc -o sr_render -D SR_RENDER sr_render.c config.c document.c navigation.c unicode.c encodings.c font.c render.c linebreak.c basics.c
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "serialreader.h"

void parsenav(char *command);
int c2nav(char c);
void save_as(const char *filename, const char *data, unsigned length);
void screen_to_pgm(const char *filename, const char *scr);

unsigned char *screen;

int main(int argc, char **argv)
{
    char command[16];

    if( argc != 2 ) {
        fprintf(stderr, "usage: sr_render <text.txt/.html>\n");
        return 1;
    }
    if( to_docdir(argv[1]) )
        return 2;
    if( read_config() )
        return 3;
    if( load_all_fonts() )
        return 4;
    screen= malloc(SCREEN_WIDTH * SCREEN_HEIGHT);
    if( !screen )
        return 5;
    if( change_file(NAV_INIT) )
        return 6;
    do {
        printf("displaying %s lines %d to %d; history %c%c\n",
                document->filename,
                document->curr_first_line, document->curr_last_line,
                (int)(have_history(NAV_PREV)? 'B':'b'),
                (int)(have_history(NAV_NEXT)? 'F':'f') );
        screen_to_pgm("/tmp/sr_render.pgm", screen);
        fgets(command, 16, stdin);
        parsenav(command);
    }
    while( *command !='q' );
    write_config();
    return 0;
}


void parsenav(char *command)
{
    int nav, num;

    if( isdigit(*command) ) {
        num= strtoul(command, &command, 10);
        if( *command == 'l' )
            change_page(NAV_SET + num);
        else if( *command == 'f' )
            change_file(NAV_SET + num);
        else if( near_posbar(num, 0) )
            posbar_page(num);
    }
    else if( *command == '+' )
        change_fontsize(NAV_NEXT);
    else if( *command == '-' )
        change_fontsize(NAV_PREV);
    else if( tolower(*command) == 'f' )
        nav_history(NAV_NEXT);
    else if( tolower(*command) == 'b' )
        nav_history(NAV_PREV);
    else if( tolower(*command) == 's' )
        nav_history(NAV_FIRST);
    else if( tolower(*command) == 'e' )
        nav_history(NAV_LAST);
    else {
        nav= c2nav(*command);
        if( nav >= 0 )
            if( isupper(*command) )
                change_file(nav);
            else
                change_page(nav);
    }
}


int c2nav(char c)
{
    switch( tolower(c) ) {
        case 'n':return NAV_NEXT;
        case 'p':return NAV_PREV;
        case 'i':return NAV_FIRST;
        case 'l':return NAV_LAST;
        default:return -1;
    }
}


void save_as(const char *filename, const char *data, unsigned length)
{
    FILE *out;

    out= fopen(filename, "w");
    if( !out ) {
        fprintf(stderr, "Could not create output file `%s'.\n", filename);
        return;
    }
    fwrite(data, 1, length, out);
    if( ferror(out) )
        fprintf(stderr, "Error writing to `%s'.\n", filename);
    fclose(out);
}


void screen_to_pgm(const char *filename, const char *scr)
{
    FILE *out;

    out= fopen(filename, "w");
    if( !out ) {
        fprintf(stderr, "Could not create output file `%s'.\n", filename);
        return;
    }
    fprintf(out, "P5\n768 1024\n255\n");
    fwrite(scr, 1, SCREEN_WIDTH * SCREEN_HEIGHT, out);
    if( ferror(out) )
        fprintf(stderr, "Error writing to `%s'.\n", filename);
    fclose(out);
}


